@@ -36,8 +36,8 @@ version_check "autoconf" "$AUTOCONF autoconf autoconf259 autoconf257 autoconf-2.
 version_check "automake" "$AUTOMAKE automake automake-1.9 automake19 automake-1.8 automake18 automake-1.7 automake17 automake-1.6 automake16" \
               "ftp://ftp.gnu.org/pub/gnu/automake/" 1 7 || DIE=1
 version_check "autopoint" "autopoint" \
-              "ftp://ftp.gnu.org/pub/gnu/gettext/" 0 11 5 || DIE=1
-version_check "libtoolize" "libtoolize libtoolize15" \
+              "ftp://ftp.gnu.org/pub/gnu/gettext/" 0 17 || DIE=1
+version_check "libtoolize" "libtoolize libtoolize15 glibtoolize" \
               "ftp://ftp.gnu.org/pub/gnu/libtool/" 1 5 0 || DIE=1
 version_check "pkg-config" "" \
               "http://www.freedesktop.org/software/pkgconfig" 0 8 0 || DIE=1
@@ -84,7 +84,7 @@ echo timestamp > stamp-h.in 2> /dev/null
 
 tool_run "$autoconf"
 debug "automake: $automake"
-tool_run "$automake" "--add-missing --copy"
+tool_run "$automake" "--add-missing --copy -Wno-portability"
 
 test -n "$NOCONFIGURE" && {
   echo "skipping configure stage for package $package, as requested."
