@@ -136,11 +136,24 @@ typedef struct _GstBufferClass GstBufferClass;
  * GST_BUFFER_MALLOCDATA:
  * @buf: a #GstBuffer.
  *
- * A pointer to any data allocated for this buffer using malloc(). If this is 
+ * A pointer to any data allocated for this buffer using g_malloc(). If this is 
  * non-NULL, this memory will be freed at the end of the buffer's lifecycle 
  * (i.e. when its refcount becomes zero).
  */
 #define GST_BUFFER_MALLOCDATA(buf)		(GST_BUFFER_CAST(buf)->malloc_data)
+/**
+ * GST_BUFFER_FREE_FUNC:
+ * @buf: a #GstBuffer.
+ *
+ * A pointer to a function that will be called on the buffer's malloc_data when
+ * this buffer is finalized. Defaults to g_free().
+ *
+ * Note that the free function only affects the buffer's malloc_data; if the
+ * buffer's malloc_data is %NULL, the function will not be called.
+ *
+ * Since: 0.10.22
+ */
+#define GST_BUFFER_FREE_FUNC(buf)		(GST_BUFFER_CAST(buf)->free_func)
 
 /**
  * GST_BUFFER_OFFSET_NONE:
@@ -198,7 +211,8 @@ typedef struct _GstBufferClass GstBufferClass;
  * network source.
  * @GST_BUFFER_FLAG_IN_CAPS: the buffer has been added as a field in a #GstCaps.
  * @GST_BUFFER_FLAG_GAP: the buffer has been created to fill a gap in the
- * stream.
+ * stream and contains media neutral data (elements can switch to optimized code
+ * path that ignores the buffer content).
  * @GST_BUFFER_FLAG_DELTA_UNIT: this unit cannot be decoded independently.
  * @GST_BUFFER_FLAG_LAST: additional flags can be added starting from this flag.
  *
@@ -233,7 +247,9 @@ typedef enum {
  * @offset_end: the last offset contained in this buffer. It has the same 
  *     format as @offset.
  * @malloc_data: a pointer to the allocated memory associated with this buffer.
- *     When the buffer is freed, this data will freed with free().
+ *     When the buffer is freed, this data will freed with @free_func.
+ * @free_func: a custom function that will be called with @malloc_data, defaults
+ *     to g_free(). Since 0.10.22.
  *
  * The structure of a #GstBuffer. Use the associated macros to access the public
  * variables.
@@ -259,8 +275,10 @@ struct _GstBuffer {
 
   guint8                *malloc_data;
 
+  GFreeFunc              free_func;
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  gpointer _gst_reserved[GST_PADDING - 1];
 };
 
 struct _GstBufferClass {
@@ -356,7 +374,7 @@ gst_buffer_ref (GstBuffer * buf)
 typedef enum {
   GST_BUFFER_COPY_FLAGS      = (1 << 0),
   GST_BUFFER_COPY_TIMESTAMPS = (1 << 1),
-  GST_BUFFER_COPY_CAPS       = (1 << 2),
+  GST_BUFFER_COPY_CAPS       = (1 << 2)
 } GstBufferCopyFlags;
 
 /**
@@ -454,9 +472,6 @@ GstBuffer*	gst_buffer_span			(GstBuffer *buf1, guint32 offset, GstBuffer *buf2,
  */
 #define		gst_value_get_buffer(v)		GST_BUFFER_CAST (gst_value_get_mini_object(v))
 
-/* --- protected --- */
-void		_gst_buffer_initialize		(void);
-
 G_END_DECLS
 
 #endif /* __GST_BUFFER_H__ */
