@@ -39,6 +39,7 @@
 /* for constructing an entry name */
 #include "gstelement.h"
 #include "gstpad.h"
+#include "gstinfo.h"
 
 /* Index signals and args */
 enum
@@ -54,6 +55,9 @@ enum
       /* FILL ME */
 };
 
+GST_DEBUG_CATEGORY_STATIC (index_debug);
+#define GST_CAT_DEFAULT index_debug
+
 static void gst_index_class_init (GstIndexClass * klass);
 static void gst_index_init (GstIndex * index);
 static void gst_index_finalize (GObject * object);
@@ -142,6 +146,9 @@ gst_index_get_type (void)
 
     index_type =
         g_type_register_static (GST_TYPE_OBJECT, "GstIndex", &index_info, 0);
+
+    GST_DEBUG_CATEGORY_INIT (index_debug, "GST_INDEX", GST_DEBUG_BOLD,
+        "Generic indexing support");
   }
   return index_type;
 }
