@@ -108,7 +108,7 @@
 #include "gst_private.h"
 #include "gstutils.h"
 #include "gstsegment.h"
-#ifdef HAVE_VALGRIND
+#ifdef HAVE_VALGRIND_H
 #  include <valgrind/valgrind.h>
 #endif
 #include <glib/gprintf.h>       /* g_sprintf */
@@ -237,7 +237,7 @@ GstDebugCategory *GST_CAT_QOS = NULL;
 
 /* FIXME: export this? */
 gboolean
-__gst_in_valgrind (void)
+_priv_gst_in_valgrind (void)
 {
   static enum
   {
@@ -248,7 +248,7 @@ __gst_in_valgrind (void)
   in_valgrind = GST_VG_UNCHECKED;
 
   if (in_valgrind == GST_VG_UNCHECKED) {
-#ifdef HAVE_VALGRIND
+#ifdef HAVE_VALGRIND_H
     if (RUNNING_ON_VALGRIND) {
       GST_CAT_INFO (GST_CAT_GST_INIT, "we're running inside valgrind");
       printf ("GStreamer has detected that it is running inside valgrind.\n");
@@ -277,14 +277,11 @@ __gst_in_valgrind (void)
 void
 _gst_debug_init (void)
 {
-  GTimeVal current;
-
-  gst_atomic_int_set (&__default_level, GST_LEVEL_DEFAULT);
-  gst_atomic_int_set (&__use_color, 1);
+  g_atomic_int_set (&__default_level, GST_LEVEL_DEFAULT);
+  g_atomic_int_set (&__use_color, 1);
 
   /* get time we started for debugging messages */
-  g_get_current_time (&current);
-  _priv_gst_info_start_time = GST_TIMEVAL_TO_TIME (current);
+  _priv_gst_info_start_time = gst_util_get_timestamp ();
 
 #ifdef HAVE_PRINTF_EXTENSION
   register_printf_function (GST_PTR_FORMAT[0], _gst_info_printf_extension_ptr,
@@ -361,7 +358,7 @@ _gst_debug_init (void)
 
 
   /* print out the valgrind message if we're in valgrind */
-  __gst_in_valgrind ();
+  _priv_gst_in_valgrind ();
 }
 
 /* we can't do this further above, because we initialize the GST_CAT_DEFAULT struct */
@@ -415,11 +412,24 @@ gst_debug_log_valist (GstDebugCategory * category, GstDebugLevel level,
   LogFuncEntry *entry;
   GSList *handler;
 
+#ifdef _MSC_VER
+  gchar *file_basename;
+#endif
+
   g_return_if_fail (category != NULL);
   g_return_if_fail (file != NULL);
   g_return_if_fail (function != NULL);
   g_return_if_fail (format != NULL);
 
+#ifdef _MSC_VER
+  /*
+   * The predefined macro __FILE__ is always the exact path given to the
+   * compiler with MSVC, which may or may not be the basename.  We work
+   * around it at runtime to improve the readability.
+   */
+  file = file_basename = g_path_get_basename (file);
+#endif
+
   message.message = NULL;
   message.format = format;
   G_VA_COPY (message.arguments, args);
@@ -433,6 +443,10 @@ gst_debug_log_valist (GstDebugCategory * category, GstDebugLevel level,
   }
   g_free (message.message);
   va_end (message.arguments);
+
+#ifdef _MSC_VER
+  g_free (file_basename);
+#endif
 }
 
 /**
@@ -576,15 +590,14 @@ gchar *
 gst_debug_construct_term_color (guint colorinfo)
 {
   GString *color;
-  gchar *ret;
 
   color = g_string_new ("\033[00");
 
   if (colorinfo & GST_DEBUG_BOLD) {
-    g_string_append (color, ";01");
+    g_string_append_len (color, ";01", 3);
   }
   if (colorinfo & GST_DEBUG_UNDERLINE) {
-    g_string_append (color, ";04");
+    g_string_append_len (color, ";04", 3);
   }
   if (colorinfo & GST_DEBUG_FG_MASK) {
     g_string_append_printf (color, ";3%1d", colorinfo & GST_DEBUG_FG_MASK);
@@ -593,11 +606,9 @@ gst_debug_construct_term_color (guint colorinfo)
     g_string_append_printf (color, ";4%1d",
         (colorinfo & GST_DEBUG_BG_MASK) >> 4);
   }
-  g_string_append (color, "m");
+  g_string_append_c (color, 'm');
 
-  ret = color->str;
-  g_string_free (color, FALSE);
-  return ret;
+  return g_string_free (color, FALSE);
 }
 
 /**
@@ -629,7 +640,6 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
   gchar pidcolor[10];
   const gchar *levelcolor;
   gint pid;
-  GTimeVal now;
   GstClockTime elapsed;
   gboolean free_color = TRUE;
   gboolean free_obj = TRUE;
@@ -669,23 +679,28 @@ gst_debug_log_default (GstDebugCategory * category, GstDebugLevel level,
     free_obj = FALSE;
   }
 
-  g_get_current_time (&now);
-  elapsed = GST_TIMEVAL_TO_TIME (now) - _priv_gst_info_start_time;
-
-  /*
-     g_printerr ("%s (%p - %" GST_TIME_FORMAT ") %s%20s%s(%s%5d%s) %s%s(%d):%s:%s%s %s\n",
-     gst_debug_level_get_name (level), g_thread_self (),
-     GST_TIME_ARGS (elapsed), color,
-     gst_debug_category_get_name (category), clear, pidcolor, pid, clear,
-     color, file, line, function, obj, clear, gst_debug_message_get (message));
-   */
+  elapsed = GST_CLOCK_DIFF (_priv_gst_info_start_time,
+      gst_util_get_timestamp ());
 
+#if defined (GLIB_SIZEOF_VOID_P) && GLIB_SIZEOF_VOID_P == 8
+  /* width of %p varies depending on actual value of pointer, which can make
+   * output unevenly aligned if multiple threads are involved, hence the %14p
+   * (should really be %18p, but %14p seems a good compromise between too many
+   * white spaces and likely unalignment on my system) */
   g_printerr ("%" GST_TIME_FORMAT
-      " %s%5d%s %p %s%s%s %s%20s %s:%d:%s:%s%s %s\n", GST_TIME_ARGS (elapsed),
+      " %s%5d%s %14p %s%s%s %s%20s %s:%d:%s:%s%s %s\n", GST_TIME_ARGS (elapsed),
       pidcolor, pid, clear, g_thread_self (), levelcolor,
       gst_debug_level_get_name (level), clear, color,
       gst_debug_category_get_name (category), file, line, function, obj, clear,
       gst_debug_message_get (message));
+#else
+  g_printerr ("%" GST_TIME_FORMAT
+      " %s%5d%s %10p %s%s%s %s%20s %s:%d:%s:%s%s %s\n", GST_TIME_ARGS (elapsed),
+      pidcolor, pid, clear, g_thread_self (), levelcolor,
+      gst_debug_level_get_name (level), clear, color,
+      gst_debug_category_get_name (category), file, line, function, obj, clear,
+      gst_debug_message_get (message));
+#endif
 
   if (free_color)
     g_free (color);
@@ -784,6 +799,8 @@ gst_debug_remove_with_compare_func (GCompareFunc func, gpointer data)
   new = __log_functions;
   while ((found = g_slist_find_custom (new, data, func))) {
     if (new == __log_functions) {
+      /* make a copy when we have the first hit, so that we modify the copy and
+       * make that the new list later */
       new = g_slist_copy (new);
       continue;
     }
@@ -854,7 +871,7 @@ gst_debug_remove_log_function_by_data (gpointer data)
 void
 gst_debug_set_colored (gboolean colored)
 {
-  gst_atomic_int_set (&__use_color, colored ? 1 : 0);
+  g_atomic_int_set (&__use_color, colored ? 1 : 0);
 }
 
 /**
@@ -913,7 +930,7 @@ gst_debug_is_active (void)
 void
 gst_debug_set_default_threshold (GstDebugLevel level)
 {
-  gst_atomic_int_set (&__default_level, level);
+  g_atomic_int_set (&__default_level, level);
   gst_debug_reset_all_thresholds ();
 }
 
@@ -1051,7 +1068,7 @@ _gst_debug_category_new (const gchar * name, guint color,
   } else {
     cat->description = g_strdup ("no description");
   }
-  gst_atomic_int_set (&cat->threshold, 0);
+  g_atomic_int_set (&cat->threshold, 0);
   gst_debug_reset_threshold (cat, NULL);
 
   /* add to category list */
@@ -1109,7 +1126,7 @@ gst_debug_category_set_threshold (GstDebugCategory * category,
     __gst_debug_min = level;
   }
 
-  gst_atomic_int_set (&category->threshold, level);
+  g_atomic_int_set (&category->threshold, level);
 }
 
 /**
@@ -1334,7 +1351,7 @@ gst_debug_remove_log_function_by_data (gpointer data)
 }
 
 gboolean
-__gst_in_valgrind (void)
+_priv_gst_in_valgrind (void)
 {
   return FALSE;
 }
