@@ -3,10 +3,7 @@
 
 #include <glib-object.h>
 #include "../gstelement.h"
-
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include "../gstparse.h"
 
 typedef struct {
   GstElement *src;
@@ -31,11 +28,13 @@ struct _graph_t {
   chain_t *chain; /* links are supposed to be done now */
   GSList *links;
   GError **error;
+  GstParseContext *ctx; /* may be NULL */
+  GstParseFlags flags;
 };
 
 
-/* 
- * Memory checking. Should probably be done with gsttrace stuff, but that 
+/*
+ * Memory checking. Should probably be done with gsttrace stuff, but that
  * doesn't really work.
  * This is not safe from reentrance issues, but that doesn't matter as long as
  * we lock a mutex before parsing anyway.
@@ -70,11 +69,11 @@ static inline void
 gst_parse_unescape (gchar *str)
 {
   gchar *walk;
-  
+
   g_return_if_fail (str != NULL);
-  
+
   walk = str;
-  
+
   while (*walk) {
     if (*walk == '\\')
       walk++;
