@@ -87,6 +87,8 @@ struct _GstQueue {
 
   /* flowreturn when srcpad is paused */
   GstFlowReturn srcresult;
+  gboolean      unexpected;
+  gboolean      eos;
 
   /* the queue of data we're keeping our grubby hands on */
   GQueue *queue;
@@ -103,6 +105,8 @@ struct _GstQueue {
   GMutex *qlock;	/* lock for queue (vs object lock) */
   GCond *item_add;	/* signals buffers now available for reading */
   GCond *item_del;	/* signals space now available for writing */
+  
+  gboolean head_needs_discont, tail_needs_discont;
 };
 
 struct _GstQueueClass {
