@@ -388,10 +388,14 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
     /* check if we need to dump messages to the console */
     if (messages) {
       const GstStructure *s;
+      guint32 seqnum;
+
+      seqnum = gst_message_get_seqnum (message);
 
       s = gst_message_get_structure (message);
 
-      g_print (_("Got Message from element \"%s\" (%s): "),
+      g_print (_("Got Message #%" G_GUINT32_FORMAT
+              " from element \"%s\" (%s): "), seqnum,
           GST_STR_NULL (GST_ELEMENT_NAME (GST_MESSAGE_SRC (message))),
           gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
       if (s) {
@@ -485,23 +489,19 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
 
         gst_message_parse_state_changed (message, &old, &new, &pending);
 
-        /* debug each state change
-           GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline), GST_DEBUG_GRAPH_SHOW_ALL, "gst-launch");
-         */
-
         /* we only care about pipeline state change messages */
         if (GST_MESSAGE_SRC (message) != GST_OBJECT_CAST (pipeline))
           break;
 
-        /* debug only overall state changes
-           {
-           gchar *dump_name;
-
-           dump_name = g_strdup_printf ("gst-launch.%s",gst_element_state_get_name (new);
-           GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline), GST_DEBUG_GRAPH_SHOW_ALL, dump_name);
-           g_free (dump_name);
-           }
-         */
+        /* dump graph for pipeline state changes */
+        {
+          gchar *dump_name = g_strdup_printf ("gst-launch.%s_%s",
+              gst_element_state_get_name (old),
+              gst_element_state_get_name (new));
+          GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS (GST_BIN (pipeline),
+              GST_DEBUG_GRAPH_SHOW_ALL, dump_name);
+          g_free (dump_name);
+        }
 
         /* ignore when we are buffering since then we mess with the states
          * ourselves. */
@@ -522,7 +522,7 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         gint percent;
 
         gst_message_parse_buffering (message, &percent);
-        fprintf (stderr, _("buffering... %d  \r"), percent);
+        fprintf (stderr, "%s %d%%  \r", _("buffering..."), percent);
 
         /* no state management needed for live pipelines */
         if (is_live)
@@ -549,6 +549,12 @@ event_loop (GstElement * pipeline, gboolean blocking, GstState target_state)
         }
         break;
       }
+      case GST_MESSAGE_LATENCY:
+      {
+        fprintf (stderr, _("Redistribute latency...\n"));
+        gst_bin_recalculate_latency (GST_BIN (pipeline));
+        break;
+      }
       case GST_MESSAGE_APPLICATION:{
         const GstStructure *s;
 
@@ -747,7 +753,7 @@ main (int argc, char *argv[])
     if (caught_error) {
       fprintf (stderr, _("ERROR: pipeline doesn't want to preroll.\n"));
     } else {
-      GTimeVal tfthen, tfnow;
+      GstClockTime tfthen, tfnow;
       GstClockTimeDiff diff;
 
       fprintf (stderr, _("Setting pipeline to PLAYING ...\n"));
@@ -773,11 +779,11 @@ main (int argc, char *argv[])
         goto end;
       }
 
-      g_get_current_time (&tfthen);
+      tfthen = gst_util_get_timestamp ();
       caught_error = event_loop (pipeline, TRUE, GST_STATE_PLAYING);
-      g_get_current_time (&tfnow);
+      tfnow = gst_util_get_timestamp ();
 
-      diff = GST_TIMEVAL_TO_TIME (tfnow) - GST_TIMEVAL_TO_TIME (tfthen);
+      diff = GST_CLOCK_DIFF (tfthen, tfnow);
 
       g_print (_("Execution ended after %" G_GUINT64_FORMAT " ns.\n"), diff);
     }
